//--------------------------------------------------------------------------------------
//  The Be Book API Fetcher thingie plugin for Eddie
//  (C) Alan Ellis 1998 (alan@cgsoftware.com)
//
//  Unauthorized duplication is restricted, therefore I am authorizing you to duplicate
// this. =)
//
//
//--------------------------------------------------------------------------------------


#include <Alert.h>
#include <Bitmap.h>
#include <Roster.h>

#include "Plugin.h"

// include "BitmapsFlip.h" here instead for a slightly more psychadelic effect =)
#include "Bitmaps.h"

void BeApiFetch(PluginInterface *pluginInterface);


PluginInterface::PluginResult 
PluginMain(PluginInterface::PluginCallSelector selector, 
	PluginInterface *pluginInterface)
{
	PluginInterface::PluginResult result = PluginInterface::kIgnored;

// This is defined in the project options under "More compiler options"
// remove the: '-Wp, -DDEBUG' to prevent the strings from spewing at the
// terminal every time you press a key!  
#ifdef DEBUG	
	fprintf(stderr, "Be API Fetch: PluginInterface::PluginResult PluginMain() called\n"
                    "              Selector = %d\n", selector);
#endif

	switch (selector)
	{

		case PluginInterface::kGetPluginVersion:
		
			pluginInterface->ReturnSupportedPluginVersion();
			pluginInterface->ReturnRequiredPluginVersion();
			result = PluginInterface::kAccepted;
			break;

		case PluginInterface::kInit:			
		
			// publish the editor primitives used by the shortcuts:
			pluginInterface->RegisterHandledPrimitiveCommand("BeAPIFetch",
				"This calls up the BeBook Fetcher and hands it a selection at the cursor.");

			result = PluginInterface::kAccepted;

#ifdef DEBUG	
	fprintf(stderr, "Be API Fetch: case PluginInterface::kInit:\n");
#endif

			break;
		
		case PluginInterface::kWindowOpened:
			
			pluginInterface->SetupPluginButton(kButtonBits, kPressedButtonBits);

			result = PluginInterface::kAccepted;
			break;

		case PluginInterface::kButtonClick:
			
			BeApiFetch(pluginInterface);
			
			result = PluginInterface::kAccepted;
			break;


		case PluginInterface::kPrimitiveInvoked:
		{
			// handle a primitive from a keyboard command
			const char *primitive = pluginInterface->CurrentPrimitive();
			
			// make sure our shortcut was called
			if (strcmp(primitive, "BeAPIFetch") == 0)
			{
				// animate the button as if it was pressed by a mouse to give the user
				// a positive feedback
				pluginInterface->AnimateButtonClick();
				
				BeApiFetch(pluginInterface);
			}
			
			result = PluginInterface::kAccepted;
			break;
		}
		// appease the compiler warning gods and fall through a bunch of cases
		case PluginInterface::kUnload      :
		case PluginInterface::kQuit        :
		case PluginInterface::kButtonPress :
		case PluginInterface::kButtonDraw  :
		case PluginInterface::kArgvPassed  :
		case PluginInterface::kPulse       :
		case PluginInterface::kDraw        :
		case PluginInterface::kMessage     :
		case PluginInterface::kMouseDown   :
		case PluginInterface::kKeyDown     :
		case PluginInterface::kWindowActivated      :  
		case PluginInterface::kWindowDeactivated    :
		case PluginInterface::kDocumentTextChanged  : 	
		case PluginInterface::kDocumentOpened       :
		case PluginInterface::kDocumentClosed       :
		case PluginInterface::kDocumentStateChanged :
		case PluginInterface::kDocumentActivated    :
		case PluginInterface::kWindowClosed         :
		case PluginInterface::kGlobalSettingsSaved  :
		case PluginInterface::kDocumentSettingsSaved:
		case PluginInterface::kGlobalPrefsShowing   :
		case PluginInterface::kSyntaxColoringOff    :
		case PluginInterface::kFindQuotedString     : 
			result = PluginInterface::kIgnored;
			break;
		// something is wrong/new/unhandled
		default:
			fprintf(stderr, "Wrong/New/Unhandled case, selector value: %d\n", selector);
			result = PluginInterface::kIgnored;
			break;
	}
	return result;
}

const char *
PluginHelp(void)
{
	return "Be API Fetch";
}
//----------------------------------------------------------------------------------------------
// alan's stuff
//----------------------------------------------------------------------------------------------

char *args[] = 
{
	"application/x-witt-bebookfetcher",
	"set",
	"Value",
	"of",
	"View",  
	"0",
	"of",
	"View",
	"0",
	"of",
	"Window",
	"0", 
	"to", 
	""	
};
//--------------------------------------------------------------------
bool IsAnUpperOrLowerOrUnderbarOrNumericDigit(const char thedigit)
{
	// I do not trap for numbers as the BeBookFetcher does not find
	// things like int32 and such.
	
	// Never mind, I was having problems with BMessage::FindInt32()
	// Numbers now supported!
	
	bool result = true;
	
	if (thedigit < '0')
	{
			result = false;
	}
	else
	{
		if (thedigit > 'z')
			result = false;
		else
		{
			// I know this could be turned into one of those huuuge
			// if (!() || () && () && ()), but I feel that this is
			// infinitely more ledgable (and the compiler should
			// spit out the same binary reguardless.)
			if (thedigit < 'a')
				if (thedigit > '9')
					if (thedigit != '_')
						if (thedigit > '9')
							if(thedigit < 'A')
								result = false;
		}
	}
	return result;
}
//--------------------------------------------------------------------
int32 CreateSelection(PluginInterface *pluginInterface, int32 &start)
{
	int32 end = start;

	while(IsAnUpperOrLowerOrUnderbarOrNumericDigit(pluginInterface->CharAt(start - 1)))
		start--;	
	
	while(IsAnUpperOrLowerOrUnderbarOrNumericDigit(pluginInterface->CharAt(end)))
		end++;
		
	return (end - start);	
}
//--------------------------------------------------------------------
void BeApiFetch(PluginInterface *pluginInterface)
{
	int32 start,
		  end;
	
	pluginInterface->GetSelection(&start, &end);
	
	int32 length = end - start;
	
	// If there is no selection, attempt to create one.
	if (length == 0)
	{
		length = CreateSelection(pluginInterface, start);
	}
	
	char *selection = new char[length + 3];
	
	pluginInterface->GetText(selection + 1, start, length);
	
	// Under some circumstances it helps to have the value string surrounded
	// by " chars, so we do this.
	selection[0]          = '\"';
	selection[length + 1] = '\"';
	// And without this, Bad Things (TM) happen.
	selection[length + 2] = '\0';
	
	args[13] = selection;
	
	fprintf(stderr, "string %s\n", selection);

	be_roster->Launch( "application/x-witt-bebookfetcher" );
	
	if (length)
		//
	    // 	If hey has not had it's mime type set to the following, this will not
	    // work. Use FileType to fix this.
		//
	    // 	I think that the mime type may be wrong in the first place, if someone
	    // could let me know if it is supposed to be "application/x-vnd.amezei-hey"
		// it would be much appreciated.
		//
		be_roster->Launch( "application/x-amezei-hey", 14, args );
}
//--------------------------------------------------------------------